//===================================================
//      DAC INTERFACE
//===================================================
`timescale  1ns/1ns

module i2s #(parameter DW=0, PSCALE=1)
    (
        input       clk,        // i clock
        input       nd_i,       // i new data
        input [2*DW-1:0] x_i,   // i[] DAC sample
        output      wck_o,      // o -LDAC
        output      bck_o,      // o Conversion clock
        output      y_o         // o DAC data
    );
    `include "f_log2.vh"
    //-------------------------
    localparam UW=((log2(PSCALE)>0)?log2(PSCALE):1);
    localparam NW=log2(2*DW);
    //-------------------------
    reg run=0, v_run=0;
    reg wck=0, v_wck=0;
    reg [UW-1:0] ucnt=PSCALE-1, v_ucnt=0;
    reg [NW-1:0] bcnt=0, v_bcnt=0;
    reg [2*DW-1:0] sr=0, v_sr=0;
    //------------------------------
    wire pclk = (ucnt==PSCALE/2);
    wire nclk = (ucnt==0);
    //------------------------------
    always @*
    begin
        v_run  = run;
        v_wck  = wck;
        v_sr   = sr;
        v_bcnt = bcnt;
        v_ucnt = |ucnt? ucnt-1: PSCALE-1;
        case(run)
        0:  begin
                v_ucnt = PSCALE-1;
                v_bcnt = 0;
                v_wck  = 0;
                if (nd_i) begin
                    v_bcnt = 2*DW-1;
                    v_sr   = x_i;
                    v_run  = 1;
                end
            end
        1:  if (nclk) begin
                v_sr = sr<<1;
                if (bcnt==DW+1)
                    v_wck = 1;
                else if (bcnt==1)
                    v_wck = 0;
                // LSB is done
                if (|bcnt)
                    v_bcnt = bcnt-1;
                else if (nd_i) begin
                    v_bcnt = 2*DW-1;
                    v_sr   = x_i;
                end
                else v_run = 0;
            end
        endcase
    end // always
    //
    always @(posedge clk)
    begin
        run  <= #1 v_run;
        ucnt <= #1 v_ucnt;
        bcnt <= #1 v_bcnt;
        wck  <= #1 v_wck;
        sr   <= #1 v_sr;
    end
    //-----------------------------------------------
    // Funny trick to avoid usage of gated clock
    //-----------------------------------------------
    wire bck;
    generate begin: BCKO
        reg [1:0] q=0;
        always @(posedge clk or negedge run)
            if (!run)      q[0] <= #1 0;
            else if (nclk) q[0] <= #1 !q[0];
        wire nck  = (PSCALE%2)? ~clk: clk;
        always @(posedge nck or negedge run)
            if (!run)      q[1] <= #1 0;
            else if (pclk) q[1] <= #1 !q[1];
        assign  bck = ^q & run;
    end endgenerate
    //------------------------------------------------
    assign bck_o = bck;
    assign wck_o = wck;
    assign y_o   = sr[2*DW-1];
    //
endmodule
/* EOF */
